﻿(function () {
    angular
		.module('parkIt.directives')
		.directive('draggable', draggable);

    function draggable() {
        return {
            restrict: 'A',
            scope: {
                dropAreaClass: '@',
                dropHandler: '=?'
            },
            link: function ($scope, $element) {
                var $dropArea = $('.' + $scope.dropAreaClass),
                    areaLeft,
                    areaTop,
                    areaWidth,
                    areaHeight,

                    elementStartLeft,
                    elementStartTop,
                    elementStartPosition;

                $dropArea.show();
                areaLeft = $dropArea.offset().left;
                areaTop = $dropArea.offset().top;
                areaWidth = $dropArea.outerWidth();
                areaHeight = $dropArea.outerHeight();
                $dropArea.hide();

                $element
                    .on('touchstart', function (e) {
                        var touches = e.originalEvent.touches;

                        elementStartPosition = $element.css('position');
                        elementStartLeft = $element.css('left');
                        elementStartTop = $element.css('top');

                        $element
                            .css({ 'position': 'fixed' })
                            .addClass('draggable');

                        $dropArea.fadeIn();
                    })
                    .on('touchmove', function (e) {
                        var touches = e.originalEvent.touches;

                        $element.css({
                            'left': touches[0].clientX - $element.outerWidth() / 2,
                            'top': touches[0].clientY - $element.outerHeight() / 2
                        });

                        e.preventDefault();

                        return false;
                    })
                    .on('touchend', function (e) {
                        var elementX = $element.offset().left + $element.outerWidth() / 2,
                            elementY = $element.offset().top + $element.outerHeight() / 2

                        if (elementX >= areaLeft && elementX <= areaLeft + areaWidth &&
                            elementY >= areaTop && elementY <= areaTop + areaHeight)
                        {
                            if ($scope.dropHandler) {
                                $scope.$apply(function () {
                                    $scope.dropHandler();
                                });
                            }

                            $dropArea.hide();
                        }
                        else {
                            $dropArea.fadeOut();
                        }

                        $element
                            .css({
                                'position': elementStartPosition,
                                'left': elementStartLeft,
                                'top': elementStartTop
                            })
                            .removeClass('draggable');
                    });
            }
        };
    };
})();