﻿(function () {
    angular
		.module('parkIt.services')
		.service('tracks', ['$q', '$http', tracks]);

    function tracks($q, $http) {

        var self = this;

        self.getTracks = function () {
            var self = this,
                deferred = $q.defer();

            $http({
                method: 'GET',
                url: Environment.API + '/tracks/allParkItTracksI'
            })
            .success(function (tracks) {
                deferred.resolve(convertTracks(tracks));
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        self.getUserTracks = function (userId) {
            var self = this,
                deferred = $q.defer();

            $http({
                method: 'GET',
                url: Environment.API + '/tracks/users/' + userId
            })
            .success(function (tracks) {
                deferred.resolve(convertTracks(tracks));
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        self.getTrack = function (id) {
            var self = this,
                deferred = $q.defer();

            $http({
                method: 'GET',
                url: Environment.API + '/tracks/' + id
            })
            .success(function (track) {
                if (track) {
                    var waypoints = undefined;

                    if (track.IsExtended) {
                        waypoints = track.Waypoints.map(function (waypoint) {
                            return new GeoPoint(waypoint.Latitude, waypoint.Longitude);
                        });
                    }

                    deferred.resolve(new Track({
                        id: track.Id,
                        type: track.Type,
                        actionDay: track.ActionDay,
                        reason: track.Reason,
                        waypoints: waypoints,
                        creatorName: track.CreatorName,
                        creationDate: track.CreationDate,
                        modifierName: track.ModifierName,
                        modifyDate: track.ModifyDate,
                        isExtended: track.IsExtended
                    }));
                }

                deferred.reject();
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        self.addTrack = function (track) {
            var self = this,
                deferred = $q.defer();

            $http({
                method: 'POST',
                url: Environment.API + '/tracks',
                data: JSON.stringify(track)
            })
            .success(function (id) {
                deferred.resolve(id);
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        },

        self.editTrack = function (id, track) {
            var self = this,
                deferred = $q.defer();

            $http({
                method: 'PUT',
                url: Environment.API + '/tracks/' + id,
                data: JSON.stringify(track)
            })
            .success(function (id) {
                deferred.resolve(id);
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        self.deleteTrack = function (trackId) {
            var self = this,
                deferred = $q.defer();

            if (!trackId)
                deferred.reject();

            $http({
                method: 'DELETE',
                url: Environment.API + '/tracks/' + trackId
            })
            .success(function () {
                deferred.resolve(trackId);
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        self.setTrackHiddenState = function (id, state) {
            var self = this,
                deferred = $q.defer();

            $http({
                method: state ? 'POST' : 'DELETE',
                url: Environment.API + '/tracks/' + id + '/hiddenFlag'
            })
            .success(function () {
                deferred.resolve();
            })
            .error(function () {
                deferred.reject();
            });

            return deferred.promise;
        };

        function convertTracks(tracks) {
            var result = [];

            if (tracks && tracks.length) {
                tracks.forEach(function (track) {
                    var waypoints = track.Waypoints.map(function (waypoint) {
                        return new GeoPoint(waypoint.Latitude, waypoint.Longitude);
                    });

                    result.push(new Track({
                        id: track.Guid,
                        type: track.Type,
                        waypoints: waypoints,
                        length: track.Length,
                        isHidden: track.IsHidden,
                        actionDay: track.ActionDay,
                        reason: track.Reason
                    }));
                });
            }

            return result;
        }
    }
})();