﻿(function () {
    angular
		.module('parkIt.controllers')
		.controller('MapController', [
            '$scope',
            '$interval',
            '$routeParams',
            'geolocation',
            'tracks',
            'localization',
            'browserDetection',
            'purchases',
            'leftMenu',
            MapController]);

    function MapController(
        $scope,
        $interval,
        $routeParams,
        geolocation,
        tracks,
        localization,
        browserDetection,
        purchases,
        leftMenu)
    {

        var map = undefined,
            tilesLayer = undefined,
            hotspotsLayer = undefined,
            positioningIntervalPromise,
            zoomOptions = {
                minZoom: 10,
                maxZoom: 18
            },
            zoomAnimationProcessing = false,
            positionPlacemark = null;

        $scope.initialized = false;
        $scope.loading = false;

        $scope.localizationService = localization;
        $scope.purchases = purchases;

        $scope.positioningEnabled = false;
        $scope.positioningZoom = 18;

        $scope.canZoomIn = true;
        $scope.canZoomOut = true;

        $scope.showHid = true;

        $scope.carPosition = null;

        //#region Init

        ymaps.ready(function () {
            $scope.$apply(function () {
                geolocation.getPosition()
                    .then(
                        function (currentPosition) {
                            $scope.initMap(currentPosition.position);
                            $scope.initialized = true;
                        },
                        function () {
                            $scope.initMap(new GeoPoint(Environment.DefaultStartLat, Environment.DefaultStartLong));
                            $scope.initialized = true;
                        });
            });
        });

        $scope.initMap = function (currentPosition) {
        	var carPlacemark,
				searchControl;

            map = new ymaps.Map("map", {
                center: [currentPosition.latitude, currentPosition.longitude],
                zoom: Environment.StartZoom,
                controls: []
            });

            map.controls.add('searchControl', { float: 'right' });

            searchControl = map.controls.get('searchControl');

            searchControl.options.set('size', 'small');
            searchControl.options.set('noPlacemark', 'true');
            searchControl.options.set('useMapBounds', 'true');

            //Клик по элементу
            searchControl.events.add('click', function () {
                $scope.$apply(function () {
                    // TODO Грязный хак, но события у элемента закрытия нет
                    $scope.showHid = !$('[class$="search_layout_panel_show"]').length;
                });
            }, this);

            // Скрываем меню поиска, выбрали пункт
            searchControl.events.add('resultshow', function () {
                $scope.$apply(function () {
                    $scope.showHid = true;
                });
            }, this);

            map.events.add('mousedown', function () {
                if ($scope.positioningEnabled) {
                    $scope.$apply(function () {
                        $scope.positioningEnabled = false;
                    });
                }
            });

            if (localStorage.getItem('carPositionLat') && localStorage.getItem('carPositionLong')) {
                $scope.carPosition = [
                    localStorage.getItem('carPositionLat'),
                    localStorage.getItem('carPositionLong')
                ];
            }

            $scope.$watch('carPosition', function () {
                if ($scope.carPosition) {
                    carPlacemark = new ymaps.Placemark($scope.carPosition, {}, {
                        iconLayout: 'default#image',
                        iconImageHref: 'Images/car.svg',
                        iconImageSize: [36, 30]
                    });

                    map.geoObjects.add(carPlacemark);

                    positionPlacemark.options.set('iconImageHref', 'Images/male.svg');
                    positionPlacemark.options.set('iconImageSize', [35, 40]);
                }
                else {
                    map.geoObjects.remove(carPlacemark);

                    positionPlacemark.options.set('iconImageHref', 'Images/car.svg');
                    positionPlacemark.options.set('iconImageSize', [36, 30]);
                }
            });

            positionPlacemark = new ymaps.Placemark([currentPosition.latitude, currentPosition.longitude], {}, {
                iconLayout: 'default#image',
                iconImageHref: 'Images/' + ($scope.carPosition ? 'male.svg' : 'car.svg'),
                iconImageSize: ($scope.carPosition ? [35, 40] : [36, 30])
            });

            map.geoObjects.add(positionPlacemark);

            $interval(function () {
                geolocation.getPosition()
                    .then(function (currentPosition) {
                        positionPlacemark.geometry.setCoordinates(
                            [currentPosition.position.latitude, currentPosition.position.longitude]);
                    });
            }, 5000);
            

            tilesLayer = new ymaps.Layer(Environment.API + '/tiles/%z/%x/%y', {
                tileTransparent: true
            });

            map.layers.add(tilesLayer);
            
            updateZoomDisabledStates(Environment.StartZoom);
        };

        //#endregion

        //#region Mobile UI

        $scope.zoomIn = function () {
            if (zoomAnimationProcessing)
                return;

            var positioningEnabledState = $scope.positioningEnabled,
                currentZoom = map.getZoom(),
                nextZoom = currentZoom + 1;

            if (positioningEnabledState) {
                $scope.positioningEnabled = false;
            }

            if (currentZoom < zoomOptions.maxZoom) {
                updateZoomDisabledStates(nextZoom);

                if (positioningEnabledState) {
                    $scope.positioningZoom = nextZoom;
                }

                zoomAnimationProcessing = true;

                map.setZoom(nextZoom, { duration: 250 })
                 .then(function () {
                     zoomAnimationProcessing = false;

                     if (positioningEnabledState) {
                         $scope.$apply(function () {
                             $scope.positioningEnabled = true;
                         });
                     }
                 });
            }
        };

        $scope.zoomOut = function () {
            if (zoomAnimationProcessing)
                return;

            var positioningEnabledState = $scope.positioningEnabled,
                currentZoom = map.getZoom(),
                nextZoom = currentZoom - 1;

            if (positioningEnabledState) {
                $scope.positioningEnabled = false;
            }

            if (currentZoom > zoomOptions.minZoom) {
                updateZoomDisabledStates(nextZoom);

                if (positioningEnabledState) {
                    $scope.positioningZoom = nextZoom;
                }

                zoomAnimationProcessing = true;

                map.setZoom(nextZoom, { duration: 250 })
                 .then(function () {
                     zoomAnimationProcessing = false;

                     if (positioningEnabledState) {
                         $scope.$apply(function () {
                             $scope.positioningEnabled = true;
                         });
                     }
                 });
            }
        };

        $scope.togglePositioning = function () {
            $scope.positioningEnabled = !$scope.positioningEnabled;
            $scope.positioningZoom = 18; //TODO

            if ($scope.positioningEnabled) {
                positioningIntervalPromise = $interval(function () {
                    geolocation.getPosition()
                        .then(function (currentPosition) {
                            if ($scope.positioningEnabled) {
                                positionPlacemark.geometry.setCoordinates(
                                    [currentPosition.position.latitude, currentPosition.position.longitude]);

                                map.setCenter(
                                    [currentPosition.position.latitude, currentPosition.position.longitude],
                                    $scope.positioningZoom,
                                    { duration: 250 });
                            }
                        });
                }, 250);
            }
            else {
                $interval.cancel(positioningIntervalPromise);
            }
        };

        $scope.toggleCarPosition = function () {
            if (!$scope.carPosition) {

                $scope.loading = true;

                geolocation.getPosition()
                    .then(function (position) {
                        localStorage.setItem('carPositionLat', position.position.latitude);
                        localStorage.setItem('carPositionLong', position.position.longitude);

                        $scope.carPosition = [
                            position.position.latitude,
                            position.position.longitude
                        ];

                        $scope.loading = false;
                    }, function () {
                        $scope.loading = false;

                        alert('Не удалось вычислить местоположение');
                    });
            }
            else
            {
                $scope.carPosition = null;

                localStorage.removeItem('carPositionLat');
                localStorage.removeItem('carPositionLong');
            }
        };

        $scope.showCarHandler = function ($event) {
            if (!$scope.carPosition)
                return;

            $scope.positioningEnabled = false;

            map.setCenter($scope.carPosition,
                        $scope.positioningZoom,
                        { duration: 250 });

            $event.stopPropagation();
        }

        $scope.openMenu = function () {
            leftMenu.openMenu();
        };

        //#endregion

        function updateZoomDisabledStates(currentZoom) {
            $scope.canZoomIn = (currentZoom < zoomOptions.maxZoom);
            $scope.canZoomOut = (currentZoom > zoomOptions.minZoom);
        };
    };
})();